//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Library, How the Components Work - Part 3
//
// Date: 16th January, 2004
// Last Updated: 18th May 2013
// Author: Darren John Dwyer
// Copyright: (c) 2002-2013 Darren John Dwyer, All Rights Reserved.
//
// Description:
// The TDx_Library interface components have automatic error detection using the
// OnError() event, which can be used to detect and identify and handle DirectX
// runtime errors.
//
// This application shows how to use the OnError() event.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSBufferDesc"
#pragma link "TDx_Sound_Library_Install"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TestButtonClick(TObject *Sender)
{
Msg( "Test button was clicked, playing an example .WAV file that does not exist... 'dummy.wav'" );

// create a default DirectSound device
if (Dx_Sound1->Create(NULL))
	{
	// create a new DirectSoundBuffer and force an error by attempting to load
    // a non-existant .WAV file into it, which will trigger the
    // TDx_SoundBuffer::OnError() event (below, next function).

    Attempted_Retry = false;
	Dx_SoundBuffer1->CreateFromFile("dummy.wav", DSBufferDesc1, Dx_Sound1);

    // now, either the sound does not exist because it could not be loaded,
    // or it exists because it was loaded from 'TDx_Library.wav'. See: OnError() below
    if (Dx_SoundBuffer1->Created)
	    {
		Dx_Sound1->SetCooperativeLevel( Handle, DSSCL_PRIORITY );
        
        // play the sound once
        Dx_SoundBuffer1->Play(0,0);

		// do something while the sound is playing
		while (Dx_SoundBuffer1->Playing) ; // actually, do nothing

		// destroy the soundbuffer
		Dx_SoundBuffer1->Destroy();
	    }
    // destroy the sound device
	Dx_Sound1->Destroy();
    }
else
    // this is one way of handling errors when Dx_Sound1::Create() fails.
    switch (Dx_Sound1->ErrorValue)
        {
        case DSERR_ALLOCATED:		Msg("After> Dx_Sound1::Create(), Error==DSERR_ALLOCATED"); break;
        case DSERR_INVALIDPARAM:	Msg("After> Dx_Sound1::Create(), Error==DSERR_INVALIDPARAM"); break;
        case DSERR_NOAGGREGATION:	Msg("After> Dx_Sound1::Create(), Error==DSERR_NOAGGREGATION"); break;
        case DSERR_NODRIVER:		Msg("After> Dx_Sound1::Create(), Error==DSERR_NODRIVER"); break;
        case DSERR_OUTOFMEMORY:		Msg("After> Dx_Sound1::Create(), Error==DSERR_OUTOFMEMORY"); break;
        case TDX_ERROR:				Msg("After> Dx_Sound1::Create(), Error==TDX_ERROR"); break;
        case TDX_BADPARAMS:			Msg("After> Dx_Sound1::Create(), Error==TDX_BADPARAMS"); break;
        case TDX_NOTCREATED:		Msg("After> Dx_Sound1::Create(), Error==TDX_NOTCREATED"); break;
        case TDX_ALREADYCREATED:	Msg("After> Dx_Sound1::Create(), Error==TDX_BADPARAMS"); break;
        default:
            Msg( "----" );
            Msg( "An error occurred after calling Dx_Sound1::Create()..." );
            Msg( "... the error code: "+TDx_Sound_Library_ErrorString(Dx_Sound1->ErrorValue) );
            Msg( "... the error message: "+TDx_Sound_Library_ErrorMessage(Dx_Sound1->ErrorValue) );
        }

// display a message
Msg( "...finished playing the example .WAV file" );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Error(TObject *Sender,
	AnsiString Function, AnsiString Error, AnsiString ErrorMessage)
{
Msg( "----" );
Msg( "An error occurred in: "+Function+"..." );
Msg( "... the error code: "+Error );
Msg( "... the error message: "+ErrorMessage );

// This is another way of handling errors.
switch (Dx_SoundBuffer1->ErrorValue)
	{
    case DSERR_BUFFERLOST:
    	break; // Do nothing more, OnBufferLost() below will handle it

    case TDX_ERROR:

    	// if the error message originated in Dx_SoundBuffer1::CreateFromFile()
    	if (Function.Pos("::CreateFromFile()")!=0)
        	{
        	// try recreating the buffer using a filename that actually exists,
	        // but only do it once in case the CreateFromFile() fails and
            // triggers this event again.
	    	if (!Attempted_Retry)
	        	{
            	Msg( "----" );
		    	Msg("The CreateFromFile() failed. Attempting to recreate buffer using 'TDx_Library.wav'");
	        	Dx_SoundBuffer1->CreateFromFile( "TDx_Library.wav", DSBufferDesc1, Dx_Sound1 );
	            Attempted_Retry = true;
	            }
            }
        break;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1BufferLost(TObject *Sender)
{
// this event is called when Dx_SoundBuffer1->ErrorValue == DSERR_BUFFERLOST
// try recreating the buffer using a filename that actually exists
Msg( "----" );
Msg("In> Dx_SoundBuffer1::OnBufferLost(), recreating buffer using 'TDx_Library.wav'");
if (Dx_SoundBuffer1->CreateFromFile( "TDx_Library.wav", DSBufferDesc1, Dx_Sound1 ))
	Dx_SoundBuffer1->Play(0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::QuitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Create(TObject *Sender)
{
/* The Level parameter defines flags indicating the priority level being requested.
   in SetCooperativeLevel(HWND pHWnd, dword pLevel)

DSSCL_EXCLUSIVE         :The application has exclusive level access to the sound
                        device. All DSSCL_PRIORITY level priviledges apply plus
                        when the application has the input focus it will be the
                        only one audible.

DSSCL_NORMAL            :The application has normal level access to the sound device.
                        The format of the primary buffer may not be changed,
                        sound output is restricted to the default 8-bit sound format.

DSSCL_PRIORITY          :The application has priority level access to the sound device.
                        Priority level access allows the use of
                        the TDx_SoundBuffer::SetFormat and Compact methods.

DSSCL_WRITEPRIMARY      :The application has write access to the primary sound buffers.
                        No secondary sound buffers can be played. This cooperative level
                        cannot be set if the DSCAPS_EMULDRIVER flag is set in the
                        associated TDSCaps component.

The application must set the cooperative level by calling this method before its buffers can
be played. The recommended cooperative level is DSSCL_PRIORITY; use other priority levels
when necessary.

*/
Dx_Sound1->SetCooperativeLevel( Handle, DSSCL_EXCLUSIVE );
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
// This is not actually required.
// The TDx_Library components automatically call Destroy() from their destructor,
// which is usually ok since most of the DirectX relationships correspond 1:1 with
// the TDx_Library construction / destruction order.
if (Dx_SoundBuffer1->Created) Dx_SoundBuffer1->Destroy();
if (Dx_Sound1->Created) Dx_Sound1->Destroy();
}
//---------------------------------------------------------------------------
